// @author: iMujtaba Nazki
// Date: 30-04-2019
// Copyright: im8488 @: 2019
import 'package:flutter/material.dart';

/// ScalePreset determines the scaling of [_holder] which is the main container
/// that holds all other components within a ThreeColumnTextBar. A value of
/// [small] sets the height of the holder to 45px, [medium] sets it to 50px,
/// [large] to 55px, and [gigantic] to 90px. The label and data text fontSize are
/// also scaled proportionally to the height of the holder, both in the device
/// portrait and landscape orientation.
enum ScalePreset {
  small,
  medium,
  large,
  gigantic,
}

/// A ThreeColumnTextBar is typically used to display a textual summary bar along
/// the top of the device screen. It has 3 columns marked as c1, c2, and c3.
/// Each column in turn has two fixed rows, wherein, the first row is used to display
/// a label which defines the data that is contained within the second row. The
/// labelText size is smaller than the dataTextSize. Examples where it can be
/// used includes, to display income / expense summary in a Personal Finance
/// Management app, whether forcast for 3 days in a weather app, etc. Keep in mind
/// that it only displays the information using 'text', hence, graphics and other
/// widgets are not supported, as of yet.
class ThreeColumnTextBar extends StatelessWidget {
  final BuildContext _context;
  final String _c1LabelText;
  final String _c1DataText;
  final String _c2LabelText;
  final String _c2DataText;
  final String _c3LabelText;
  final String _c3DataText;
  final Color _labelTextColor;
  final Color _dataTextColor;
  final Color _holderColor;
  final double _holderBorderRadius;
  final ScalePreset _scalePreset;
  final bool _showHangingDivider;
  final Color _hangingDividerColor;

  /// It allows you to create the most configurable ThreeColumnTextBar with
  /// customizable default grey background. Also, by default, the label and data
  /// text are blue and black in color respectively, border radius is set to 0.0,
  /// and hanging dividers are visible. The prefix of c1, c2, and c3 represent
  /// Column #1, Column #2, and Column #3, respectively.
  ThreeColumnTextBar({
    @required BuildContext context,
    String c1LabelText = 'labelText',
    String c1DataText = 'dataText',
    String c2LabelText = 'labelText',
    String c2DataText = 'dataText',
    String c3LabelText = 'labelText',
    String c3DataText = 'dataText',
    Color labelTextColor = Colors.blue,
    Color dataTextColor = Colors.black,
    ScalePreset scalePreset = ScalePreset.small,
    Color holderColor = Colors.grey,
    double holderBorderRadius = 0.0,
    bool showHangingDivider = true,
    Color dividerColor = Colors.white,
  })  : _context = context,
        _c1LabelText = c1LabelText,
        _c1DataText = c1DataText,
        _c2LabelText = c2LabelText,
        _c2DataText = c2DataText,
        _c3LabelText = c3LabelText,
        _c3DataText = c3DataText,
        _labelTextColor = labelTextColor,
        _dataTextColor = dataTextColor,
        _holderColor = holderColor,
        _holderBorderRadius = holderBorderRadius,
        _scalePreset = scalePreset,
        _showHangingDivider = showHangingDivider,
        _hangingDividerColor = dividerColor;

  /// Constructs a ThreeColumnTextBar with transparent background and effects.
  ThreeColumnTextBar.transparent({
    @required BuildContext context,
    String c1LabelText = 'labelText',
    String c1DataText = 'dataText',
    String c2LabelText = 'labelText',
    String c2DataText = 'dataText',
    String c3LabelText = 'labelText',
    String c3DataText = 'dataText',
    ScalePreset scalePreset = ScalePreset.small,
    double holderBorderRadius = 0.0,
    bool showHangingDivider = true,
  }) : this(
          context: context,
          c1LabelText: c1LabelText,
          c1DataText: c1DataText,
          c2LabelText: c2LabelText,
          c2DataText: c2DataText,
          c3LabelText: c3LabelText,
          c3DataText: c3DataText,
          scalePreset: scalePreset,
          showHangingDivider: showHangingDivider,

          // Pre-configured.
          holderColor: Colors.transparent,
          labelTextColor: Colors.deepPurpleAccent,
          dataTextColor: Colors.black,
          dividerColor: Colors.deepPurple,
        );

  /// Constructs a ThreeColumnTextBar with dark background and effects.
  ThreeColumnTextBar.dark({
    @required BuildContext context,
    String c1LabelText = 'labelText',
    String c1DataText = 'dataText',
    String c2LabelText = 'labelText',
    String c2DataText = 'dataText',
    String c3LabelText = 'labelText',
    String c3DataText = 'dataText',
    ScalePreset scalePreset = ScalePreset.small,
    double holderBorderRadius = 0.0,
    bool showHangingDivider = true,
  }) : this(
          context: context,
          c1LabelText: c1LabelText,
          c1DataText: c1DataText,
          c2LabelText: c2LabelText,
          c2DataText: c2DataText,
          c3LabelText: c3LabelText,
          c3DataText: c3DataText,
          scalePreset: scalePreset,
          holderBorderRadius: holderBorderRadius,
          showHangingDivider: showHangingDivider,

          // Pre-configured.
          holderColor: Colors.black,
          labelTextColor: Colors.yellow,
          dataTextColor: Colors.white,
        );

  /// Constructs a ThreeColumnTextBar with deepPurple background and effects.
  ThreeColumnTextBar.deepPurple({
    @required BuildContext context,
    String c1LabelText = 'labelText',
    String c1DataText = 'dataText',
    String c2LabelText = 'labelText',
    String c2DataText = 'dataText',
    String c3LabelText = 'labelText',
    String c3DataText = 'dataText',
    ScalePreset scalePreset = ScalePreset.small,
    double holderBorderRadius = 0.0,
    bool showHangingDivider = true,
  }) : this(
          context: context,
          c1LabelText: c1LabelText,
          c1DataText: c1DataText,
          c2LabelText: c2LabelText,
          c2DataText: c2DataText,
          c3LabelText: c3LabelText,
          c3DataText: c3DataText,
          scalePreset: scalePreset,
          showHangingDivider: showHangingDivider,
          holderBorderRadius: holderBorderRadius,

          // Pre-configured.
          holderColor: Colors.deepPurple,
          labelTextColor: Colors.yellow,
          dataTextColor: Colors.white,
        );

  @override
  Widget build(BuildContext context) {
    return _holder();
  }

  /// Holder is the primary container that holds all other components within the
  /// ThreeColumnTextBar. Its height is computed based on the [ScalePreset]
  /// specified by the user and remains the same irrespective of the device
  /// orientation. It's width is the width of the device screen minus left padding
  /// of 5px and right padding of 5px. Columns within the holder are aligned to
  /// the center and cannot be configured to be aligned any other way. Also, the
  /// columns can be separated using the [_hangingVerticalDivider].
  Widget _holder() {
    return Container(
      height: _computedHolderHeight(),
      margin: EdgeInsets.all(5.0),
      decoration: BoxDecoration(
        color: _holderColor,
        borderRadius: BorderRadius.circular(_holderBorderRadius),
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          _textColumn(
            labelText: _c1LabelText,
            dataText: _c1DataText,
          ),
          _showHangingDivider ? _hangingVerticalDivider() : Container(),
          _textColumn(
            labelText: _c2LabelText,
            dataText: _c2DataText,
          ),
          _showHangingDivider ? _hangingVerticalDivider() : Container(),
          _textColumn(
            labelText: _c3LabelText,
            dataText: _c3DataText,
          ),
        ],
      ),
    );
  }

  /// A [_textColumn] represents a single column within a ThreeColumnTextBar,
  /// which is formatted to hold the label and the data Text in a top-down order.
  Widget _textColumn(
      {String labelText = 'labelText', String dataText = 'dataText'}) {
    // Column divided into 3 equal parts, 0.5 for extra padding around columns.
    final double columnWidth = MediaQuery.of(_context).size.width / 3.5;

    return Container(
      width: columnWidth,
      padding: const EdgeInsets.all(5.0),
      child: Column(
        // Center the content.
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Text(
            labelText,
            style: TextStyle(
              color: _labelTextColor,
              // Increase fontSize, when increase in tile size, and vice-versa.
              fontSize: _computedTitleSize(),
            ),
          ),
          SizedBox(
            height: 3.0,
          ),
          Text(
            dataText,
            style: TextStyle(
              color: _dataTextColor,
              // Increase fontSize, when increase in tile size, and vice-versa.
              fontSize: _computedSubTitleSize(),
              fontWeight: FontWeight.bold,
            ),
          ),
        ],
      ),
    );
  }

  /// Customized Divider that separates the columns within a ThreeColumnTextBar.
  /// The height of the divider is adjusted to give it a hanging effect, hence,
  /// the name [_hangingVerticalDivider].
  Widget _hangingVerticalDivider() {
    return SizedBox(
      height: _computedHolderHeight() / 2.0,
      child: VerticalDivider(
        color: _hangingDividerColor,
      ),
    );
  }

  /// Returns the computed [_holder] height based on [ScalePreset] specified.
  double _computedHolderHeight() {
    return _scalePreset == ScalePreset.small
        ? 45.0
        : _scalePreset == ScalePreset.medium
            ? 55.0
            : _scalePreset == ScalePreset.large ? 60.0 : 90.0;
  }

  /// Returns the computed [_titleSize] based on [ScalePreset] specified.
  double _computedTitleSize() {
    return _scalePreset == ScalePreset.small
        ? 10.0
        : _scalePreset == ScalePreset.medium
            ? 11.0
            : _scalePreset == ScalePreset.large ? 12.0 : 13.0;
  }

  /// Returns the computed [_subTitleTextSize] based on [ScalePreset] specified.
  double _computedSubTitleSize() {
    return _scalePreset == ScalePreset.small
        ? 14.0
        : _scalePreset == ScalePreset.medium
            ? 15.0
            : _scalePreset == ScalePreset.large ? 16.0 : 17.0;
  }
}
